@@ -18,6 +18,10 @@
 #'   for static or multiple values of each for moving formants. \code{formants =
 #'   NA} defaults to schwa. Time stamps for formants and mouthOpening can be
 #'   specified in ms or an any other arbitrary scale.
+#' @param interpol the method of smoothing envelopes based on provided mouth
+#'   anchors: 'approx' = linear interpolation, 'spline' = cubic spline, 'loess'
+#'   (default) = polynomial local smoothing function. NB: this does NOT affect
+#'   the smoothing of formant anchors
 #' @param formDrift scale factor regulating the effect of temperature on the
 #'   depth of random drift of all formants (user-defined and stochastic): the
 #'   higher, the more formants drift at a given temperature
@@ -70,6 +74,12 @@
 #'   vocalTract = 16, plot = TRUE, lipRad = 6, noseRad = 4,
 #'   mouthAnchors = data.frame(time = c(0, .5, 1), value = c(0, 0, .5)))
 #'
+#' # scale formant amplitude and/or bandwidth
+#' e = getSpectralEnvelope(nr = 512, nc = 50,
+#'   formants = soundgen:::convertStringToFormants('a'),
+#'   formantWidth = 2, formantDep = .5,
+#'   temperature = 0, plot = TRUE)
+#'
 #' # manual specification of formants
 #' e = getSpectralEnvelope(nr = 512, nc = 50, plot = TRUE, samplingRate = 16000,
 #'   formants = list(f1 = data.frame(time = c(0, 1), freq = c(900, 500),
@@ -82,9 +92,11 @@ getSpectralEnvelope = function(nr,
                                nc,
                                formants = NA,
                                formantDep = 1,
+                               formantWidth = 1,
                                lipRad = 6,
                                noseRad = 4,
                                mouthAnchors = NA,
+                               interpol = c('approx', 'spline', 'loess')[3],
                                mouthOpenThres = 0.2,
                                openMouthBoost = 0,
                                vocalTract = NULL,
@@ -204,15 +216,18 @@ getSpectralEnvelope = function(nr,
         # Solving for nExtraFormants gives (nyquist * 4 / formantDispersion + 1) / 2:
         nExtraFormants = round((samplingRate * 2 / formantDispersion + 1) / 2) - nFormants
         if (is.numeric(nExtraFormants) && nExtraFormants > 0) {
-          extraFreqs_regular = (2 * ((nFormants_integer + 1):(nFormants_integer + nExtraFormants)) - 1) /
+          extraFreqs_regular = (2 * ((nFormants_integer + 1):
+                                       (nFormants_integer + nExtraFormants)) - 1) /
             2 * formantDispersion
           extraFreqs = rgamma(nExtraFormants,
+                              # mean = extraFreqs_regular, sd = sdG
                               extraFreqs_regular ^ 2 / sdG ^ 2,
                               extraFreqs_regular / sdG ^ 2)
           extraAmps = rgamma(
             nExtraFormants,
-            (formantDep / temperature) ^ 2,
-            formantDepStoch * formantDep / (formantDepStoch * temperature) ^ 2
+            # mean = formantDepStoch, sd = formantDepStoch * temperature
+            1 / temperature ^ 2,
+            1 / (formantDepStoch * temperature ^ 2)
           )
           extraWidths = getBandwidth(extraFreqs)
           for (f in 1:nExtraFormants) {
@@ -257,7 +272,7 @@ getSpectralEnvelope = function(nr,
       # frequencies expressed in bin indices (how many bin widths above the
       # central frequency of the first bin)
       formants_upsampled[[f]][, 'width'] =
-        formants_upsampled[[f]][, 'width'] / bin_width
+        formants_upsampled[[f]][, 'width'] / bin_width * formantWidth
     }
 
     # mouth opening
@@ -269,6 +284,7 @@ getSpectralEnvelope = function(nr,
       mouthOpening_upsampled = getSmoothContour(
         len = nc,
         anchors = mouthAnchors,
+        interpol = interpol,
         valueFloor = permittedValues['mouthOpening', 'low'],
         valueCeiling = permittedValues['mouthOpening', 'high'],
         plot = FALSE
@@ -787,6 +803,10 @@ estimateVTL = function(formants, speedSound = 35400, checkFormat = TRUE) {
 #' @param formDrift,formDisp scaling factors for the effect of temperature on
 #'   formant drift and dispersal, respectively
 #' @param windowLength_points length of FFT window, points
+#' @param interpol the method of smoothing envelopes based on provided mouth
+#'   anchors: 'approx' = linear interpolation, 'spline' = cubic spline, 'loess'
+#'   (default) = polynomial local smoothing function. NB: this does NOT affect
+#'   the smoothing of formant anchors
 #' @inheritParams soundgen
 #' @export
 #' @examples
@@ -811,10 +831,12 @@ addFormants = function(sound,
                        vocalTract = NA,
                        formantDep = 1,
                        formantDepStoch = 20,
+                       formantWidth = 1,
                        lipRad = 6,
                        noseRad = 4,
                        mouthOpenThres = 0,
                        mouthAnchors = NA,
+                       interpol = c('approx', 'spline', 'loess')[3],
                        temperature = 0.025,
                        formDrift = 0.3,
                        formDisp = 0.2,
@@ -827,6 +849,11 @@ addFormants = function(sound,
     # otherwise fft glitches
     soundFiltered = sound
   } else {
+    # pad input with one windowLength_points of 0 to avoid softening the attack
+    sound = c(rep(0, windowLength_points),
+              sound,
+              rep(0, windowLength_points))
+
     # for very short sounds, make sure the analysis window is no more
     #   than half the sound's length
     windowLength_points = min(windowLength_points, floor(length(sound) / 2))
@@ -854,10 +881,12 @@ addFormants = function(sound,
       formants = formants,
       formantDep = formantDep,
       formantDepStoch = formantDepStoch,
+      formantWidth = formantWidth,
       lipRad = lipRad,
       noseRad = noseRad,
       mouthOpenThres = mouthOpenThres,
       mouthAnchors = mouthAnchors,
+      interpol = interpol,
       temperature = temperature,
       formDrift = formDrift,
       formDisp = formDisp,
@@ -908,7 +937,198 @@ addFormants = function(sound,
     )
     soundFiltered = soundFiltered / max(soundFiltered) # normalize
   }
+
+  # remove zero padding
+  l = length(soundFiltered)
+  tailIdx = suppressWarnings(max(which(soundFiltered[(l - windowLength_points):l] > .03)))
+  idx = l - windowLength_points + tailIdx - 1
+  if(!is.finite(idx)) idx = l - windowLength_points
+  soundFiltered = soundFiltered[(windowLength_points + 1):idx]
+
   # spectrogram(soundFiltered, samplingRate = samplingRate)
   # playme(soundFiltered, samplingRate = samplingRate)
   return(soundFiltered)
 }
+
+
+#' Schwa-related formant conversion
+#'
+#' This function performs several types of conceptually related conversion of
+#' formant frequencies in relation to the neutral schwa sound based on the
+#' one-tube model of the vocal tract. Case 1: if we know vocal tract length
+#' (VTL) but not formant frequencies, \code{schwa()} estimates formants
+#' corresponding to a neutral schwa sound in this vocal tract, assuming that it
+#' is perfectly cylindrical. Case 2: if we know the frequencies of a few lower
+#' formants, \code{schwa()} estimates the deviation of observed formant
+#' frequencies from the neutral values expected in a perfectly cylindrical vocal
+#' tract (based on the VTL as specified or as estimated from formant
+#' dispersion). Case 3: if we want to geneate a sound with particular relative
+#' formant frequencies (e.g. high F1 and low F2 relative to the schwa for this
+#' vocal tract), \code{schwa()} calculates the corresponding formant frequencies
+#' in Hz. See examples below for an illustration of these three suggested uses.
+#'
+#' Algorithm: the expected formant dispersion is given by \eqn{speedSound / (2 *
+#' vocalTract)}, and F1 is expected at half the value of formant dispersion. See
+#' e.g. Stevens (2000) "Acoustic phonetics", p. 139. Basically, we estimate
+#' vocal tract length and see if each formant is higher or lower than expected
+#' for this vocal tract. For this to work, we have to know either the
+#' frequencies of enough formants (not just the first two) or the true length of
+#' the vocal tract.
+#' @return Returns a list with the following components: \describe{
+#'   \item{vtl_measured}{VTL as provided by the user, cm}
+#'   \item{vocalTract_apparent}{VTL estimated based on formants frequencies
+#'   provided by the user, cm} \item{ff_measured}{formant frequencies as
+#'   provided by the user, Hz} \item{ff_schwa}{formant frequencies corresponding
+#'   to a neutral schwa sound in this vocal tract, Hz}
+#'   \item{ff_theoretical}{formant frequencies corresponding to the
+#'   user-provided relative formant frequencies, Hz}
+#'   \item{ff_relative}{deviation of formant frequencies from those expected for
+#'   a schwa, \% (e.g. if the first ff_relative is -25, it means that F1 is 25\%
+#' lower than expected for a schwa in this vocal tract)} }
+#' @param formants a numeric vector of observed (measured) formant frequencies,
+#'   Hz
+#' @param vocalTract the length of vocal tract, cm
+#' @param formants_relative a numeric vector of target relative formant
+#'   frequencies, \% deviation from schwa (see examples)
+#' @param nForm the number of formants to estimate (integer)
+#' @inheritParams getSpectralEnvelope
+#' @export
+#' @examples
+#' ## CASE 1: known VTL
+#' # If vocal tract length is known, we calculate expected formant frequencies
+#' schwa(vocalTract = 17.5)
+#' schwa(vocalTract = 13, nForm = 5)
+#'
+#' ## CASE 2: known (observed) formant frequencies
+#' # Let's take formant frequencies in three vocalizations
+#' #       (/a/, /i/, /roar/) by the same male speaker:
+#' formants_a = c(860, 1430, 2900, 4200, 5200)
+#' s_a = schwa(formants = formants_a)
+#' s_a
+#' # We get an estimate of VTL (s_a$vtl_apparent = 15.2 cm),
+#' #   same as with estimateVTL(formants_a)
+#' # We also get theoretical schwa formants: s_a$ff_schwa
+#' # And we get the difference (%) in observed vs expected
+#' #   formant frequencies: s_a$ff_relative
+#' # [a]: F1 much higher than expected, F2 slightly lower
+#'
+#' formants_i = c(300, 2700, 3400, 4400, 5300, 6400)
+#' s_i = schwa(formants = formants_i)
+#' s_i
+#' # The apparent VTL is slightly smaller (14.5 cm)
+#' # [i]: very low F1, very high F2
+#'
+#' formants_roar = c(550, 1000, 1460, 2280, 3350,
+#'                   4300, 4900, 5800, 6900, 7900)
+#' s_roar = schwa(formants = formants_roar)
+#' s_roar
+#' # Note the enormous apparent VTL (22.5 cm!)
+#' # (lowered larynx and rounded lips exaggerate the apparent size)
+#' # s_roar$ff_relative: high F1 and low F2-F4
+#'
+#' schwa(formants = formants_roar[1:4])
+#' # based on F1-F4, apparent VTL is almost 28 cm!
+#' # Since the lowest formants are the most salient,
+#' # the apparent size is exaggerated even further
+#'
+#' # If you know VTL, a few lower formants are enough to get
+#' #   a good estimate of the relative formant values:
+#' schwa(formants = formants_roar[1:4], vocalTract = 19)
+#' # NB: in this case theoretical and relative formants are calculated
+#' #  based on user-provided VTL (vtl_measured) rather than vtl_apparent
+#'
+#' ## CASE 3: from relative to absolute formant frequencies
+#' # Say we want to generate a vowel sound with F1 20% below schwa
+#' #    and F2 40% above schwa, with VTL = 15 cm
+#' s = schwa(formants_relative = c(-20, 40), vocalTract = 15)
+#' # s$ff_schwa gives formant frequencies for a schwa, while
+#' #   s$ff_theoretical gives formant frequencies for a sound with
+#' #   target relative formant values (low F1, high F2)
+#' schwa(formants = s$ff_theoretical)
+schwa = function(formants = NULL,
+                 vocalTract = NULL,
+                 formants_relative = NULL,
+                 nForm = 8,
+                 speedSound = 35400) {
+  # check input
+  if (is.null(formants) & is.null(vocalTract)) {
+    stop('Please pecify formant frequencies and/or vocal tract length')
+  }
+  if (!is.null(formants) && (!is.numeric(formants) | any(formants < 0))) {
+    stop('formants must be positive numbers (Hz)')
+  }
+  if (!is.null(formants_relative) && !is.numeric(formants_relative)) {
+    stop('formants_relative must be positive numbers (Hz)')
+  }
+  if (!is.null(vocalTract) && (!is.numeric(vocalTract) | vocalTract < 0)) {
+    stop('vocalTract must be a positive number (cm)')
+  }
+  if (!is.null(formants_relative) && is.null(vocalTract)) {
+    stop('vocalTract must be specified to convert relative formants to Hz')
+  }
+
+  if (is.null(formants_relative)) {
+    ## we don't know and want to calculate formants_relative
+    # calculate formant dispersion and apparent VTL
+    if (is.null(formants)) {
+      # we don't know formants, we know VTL
+      formantDispersion = speedSound / (2 * vocalTract)
+      vocalTract_apparent = NULL
+    } else {
+      # we know formants
+      if (is.null(vocalTract)) {
+        # we don't know VTL
+        formantDispersion = getFormantDispersion(formants,
+                                                 speedSound = speedSound,
+                                                 method = 'accurate')
+        vocalTract_apparent = speedSound / (2 * formantDispersion)
+      } else {
+        # we know VTL
+        formantDispersion_apparent = getFormantDispersion(
+          formants,
+          speedSound = speedSound,
+          method = 'accurate'
+        )
+        formantDispersion = speedSound / (2 * vocalTract)
+        vocalTract_apparent = speedSound / (2 * formantDispersion_apparent)
+      }
+    }
+
+    # calculate relative formant frequencies
+    if (is.null(formants)) {
+      idx = 1:nForm
+    } else {
+      idx = 1:length(formants)
+    }
+    ff_schwa = (2 * idx - 1) / 2 * formantDispersion
+    ff_relative = (formants / ff_schwa - 1) * 100
+    ff_theoretical = NULL
+  } else {
+    ## we know formants_relative and vocalTract and want to convert ff to Hz
+
+    # make sure formants_relative is at least nForm long
+    if (length(formants_relative) < nForm) {
+      formants_relative = c(formants_relative,
+                            rep(0, nForm - length(formants_relative)))
+    }
+
+    # calculate formants in Hz
+    formantDispersion = speedSound / (2 * vocalTract)
+    idx = 1:length(formants_relative)
+    ff_schwa = (2 * idx - 1) / 2 * formantDispersion
+    ff_theoretical = ff_schwa * (1 + formants_relative / 100)
+    vocalTract_apparent = NULL
+    ff_relative = formants_relative
+  }
+
+  # prepare the output
+  out = list(vtl_measured = vocalTract,
+             vtl_apparent = vocalTract_apparent,
+             ff_measured = formants,
+             ff_schwa = ff_schwa,
+             ff_theoretical = ff_theoretical,
+             ff_relative = ff_relative)
+  # do not return empty elements
+  out = out[lapply(out, length) > 0]
+  return(out)
+}
