@@ -47,6 +47,7 @@ permittedValues = matrix(c(
   'mouthOpenThres', 0, 0, 1, .05,
   'formantDep', 1, 0, 2, .1,
   'formantDepStoch', 20, 0, 40, 5,
+  'formantWidth', 1, .1, 10, .1,
   'subFreq', 100, 10, 1000, 10,
   'subDep', 100, 0, 500, 10,
   'shortestEpoch', 300, 50, 500, 25,
@@ -110,6 +111,7 @@ defaults = list(
   mouthOpenThres = 0,
   formantDep = 1,
   formantDepStoch = 20,
+  formantWidth = 1,
   subFreq = 100,
   subDep = 100,
   shortestEpoch = 300,
